"""Takes a bunch of Rougail XML dispatched in differents folders
as an input and outputs a Tiramisu's file.

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Sample usage::

    >>> from rougail import Rougail
    >>> rougail = Rougail()
    >>> rougail.load_dictionaries(['/usr/share/rougail/dicos'])
    >>> rougail.load_extra_dictionaries('extra1', ['/usr/share/rougail/extra1'])
    >>> rougail.space_visitor('/usr/share/rougail/funcs.py')
    >>> tiramisu = rougail.save()

The Rougail

- loads the XML into an internal RougailObjSpace representation
- visits/annotates the objects
- dumps the object space as Tiramisu string

The visit/annotation stage is a complex step that corresponds to the Rougail
procedures.
"""

from typing import List

from .i18n import _
from .config import RougailConfig
from .objspace import RougailObjSpace
from .xmlreflector import XMLReflector
from .tiramisureflector import TiramisuReflector
from .annotator import SpaceAnnotator
from .error import DictConsistencyError


class RougailConvert:
    """Rougail object
    """
    def __init__(self) -> None:
        self.xmlreflector = XMLReflector()
        self.xmlreflector.parse_dtd()
        self.rougailobjspace = RougailObjSpace(self.xmlreflector)
        self._load_dictionaries(None, RougailConfig['dictionaries_dir'])
        for extra_name, extra_dir in RougailConfig['extra_dictionaries'].items():
            self._load_dictionaries(extra_name, extra_dir)

    def _load_dictionaries(self,
                           namespace: str,
                           xmlfolders: List[str],
                           ) -> List[str]:
        if namespace in ['services', RougailConfig['variable_namespace']]:
            msg = _(f'Namespace name "{namespace}" is not allowed')
            raise DictConsistencyError(msg, 21, None)
        if hasattr(self.rougailobjspace.space, 'variables') and \
                namespace in self.rougailobjspace.space.variables:
            msg = _(f'Namespace "{namespace}" is already loader')
            raise DictConsistencyError(msg, 2, None)
        for xmlfile in self.xmlreflector.load_xml_from_folders(xmlfolders):
            document = self.xmlreflector.parse_xmlfile(xmlfile)
            self.rougailobjspace.redefine_variables = []
            self.rougailobjspace.xml_parse_document(xmlfile,
                                                    document,
                                                    namespace,
                                                    )

    def save(self,
             filename: str,
             ) -> str:
        """Return tiramisu object declaration as a string
        """
        functions_file = RougailConfig['functions_file']
        SpaceAnnotator(self.rougailobjspace,
                       functions_file,
                       )
        tiramisu_objects = TiramisuReflector(self.rougailobjspace,
                                             functions_file,
                                             )
        self.rougailobjspace = None
        output = tiramisu_objects.get_text() + '\n'
        if filename:
            with open(filename, 'w') as tiramisu:
                tiramisu.write(output)
        return output
